#
# MIT License
#
# Copyright (c) 2023 Wilhelm Ågren
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# File created: 2023-01-23
# Last updated: 2023-01-31
#

import logging
import sys
import requests

strformat = '%(asctime)s %(message)s'
logging.basicConfig(
    level=logging.INFO,
    format=strformat,
)
logger = logging.getLogger(__name__)
logger.addHandler(logging.StreamHandler(sys.stdout))

__all__ = (
    'BaseAPI',
    'BaseLeagueAPI',
    'BaseValorantAPI',
)

class BaseAPI(object):
    """
    Implementation of fundamental api object functionality. The BaseAPI is 
    inherited by specific game api, which in turn are inherited by specific
    endpoint implementations. For a full list of all available api's, 
    take a look at the official Riot Games developer documentation.
    
    Parameters
    ----------
    game: str
        A string specifying what game the api is connected to.
    api: str
        A string specifying the name of the endpoint, Riot Games api.
    platform: str
        A string specifying what platform the requests are made to.
        For example, `euw1` for League of Legends.
    region: str
        A strig specifying what region the requests are made to.
        For exampl,e 'europe', for League of Legends.

    """

    def __init__(self, game, api, platform, region) -> None:
        self._game = game
        self._api = api
        self._platform = platform
        self._region = region

    @property
    def platform(self) -> str:
        """ Return the set API platform. """
        return self._platform
    
    @property
    def region(self) -> str:
        """ Return the set API region. """
        return self._region

    @property
    def params(self) -> dict:
        """ Return the internal parameter dictionary. """
        return self._params

    def set_params(self, **kwargs) -> None:
        """
        Set the internal parameter dictionary with the provided key-value pairs.
        For example, `region`: `europe`, or `platform`: `euw1`.

        Parameters
        ----------
        **kwargs: dict
            The dictionary of parameters which is later used in the substitution
            during the GET request. Keys are subtituted for corresponding values.

        """
        logging.info(f'setting params: {kwargs}')
        params = {}
        for k, value in kwargs.items():
            key = '{' + f'{k}' + '}'
            params[key] = value

        self._params = params
    
    def get(self, token, endpoint) -> requests.Response:
        """
        Perform a GET request to the REST endpoint provided by Riot Games.
        The token required is never stored internally, as it is personal.

        Parameters
        ----------
        token: str
            The token identifying your access rights to the Riot Games
            developer api, personal, provided at runtime to Hook.
        endpoint: str
            The api endpoint to perform a GET request to.
        
        Returns
        -------
        The HTTP response from the GET request to the endpoint. See
        requests.Response for all possible HTTP response codes.

        """
        url = f'https://{endpoint}?api_key={token}'

        for key, value in self.params.items():
            if value is not None:
                url = url.replace(key, str(value))
        
        logging.info(f'GET {url}')
        response = requests.get(url)

        return response

class BaseLeagueAPI(BaseAPI):
    """ Define parent class for all League of legends endpoints. """
    def __init__(self, api, platform, region) -> None:
        super().__init__('league', api, platform, region)

class BaseValorantAPI(BaseAPI):
    """ Define parent class for all Valorant endpoints. """
    def __init__(self, api, platform, region) -> None:
        super().__init__('valorant', api, platform, region)

